use std::rc::Rc;
use crate::vm::object_type::DmObjectType;

#[derive(Debug, PartialEq, Eq)]
pub enum DmType {
    Primitive(DmPrimitiveType),
    Object(Rc<DmObjectType>),
}

#[derive(Debug, PartialEq, Eq)]
pub enum DmPrimitiveType {
    Byte,
    Int,
    Long,
    Double,
    Boolean,
    Pointer(Rc<DmType>),
    ByteArray(usize),
    IntArray(usize),
    LongArray(usize),
    DoubleArray(usize),
    BooleanArray(usize),
    PointerArray(usize, Rc<DmType>),
    Unit,
}

impl DmPrimitiveType {
    pub fn size_in_bytes(&self) -> usize {
        match self {
            DmPrimitiveType::Byte => size_of::<u8>(),
            DmPrimitiveType::Int => size_of::<i32>(),
            DmPrimitiveType::Long => size_of::<i64>(),
            DmPrimitiveType::Double => size_of::<f64>(),
            DmPrimitiveType::Boolean => size_of::<bool>(),
            DmPrimitiveType::Pointer(_) => size_of::<usize>(),
            DmPrimitiveType::ByteArray(length) => *length * size_of::<u8>(),
            DmPrimitiveType::IntArray(length) => *length * size_of::<i32>(),
            DmPrimitiveType::LongArray(length) => *length * size_of::<i64>(),
            DmPrimitiveType::DoubleArray(length) => *length * size_of::<f64>(),
            DmPrimitiveType::BooleanArray(length) => *length * size_of::<bool>(),
            DmPrimitiveType::PointerArray(length, _) => *length * size_of::<usize>(),
            DmPrimitiveType::Unit => todo!("Need to determine size of Unit... is it dependent on the size of the thing which is Unit?")
        }
    }
}
