use crate::push_bytes;

/// ## mov(register: u8, operand: u32)
/// - 0: opcode
/// - 1: register
/// - 2..5: operand
pub const MOV_INT: u8 = 0x01;

pub const MOV_LONG: u8 = 0x02;
pub const MOV_DOUBLE: u8 = 0x03;

/// ## mov(target_register: u8, source_register: u8)
/// 0: opcode
/// 1: target_register
/// 2: source_register
pub const MOV_REGISTER: u8 = 0x04;

/// ## alloc(register: u8, size: u32)
/// 0: opcode
/// 1: register
/// 2..5: size
pub const ALLOC: u8 = 0x05;

/// ## dealloc(register: u8)
/// 0: opcode
/// 1: register
pub const DEALLOC: u8 = 0x06;

/// ## mov_int_to(register: u8, offset: u32, operand: u32)
/// 0: opcode
/// 1: register
/// 2..5: offset
/// 6..9: operand
pub const MOV_INT_TO: u8 = 0x07;

pub const MOV_LONG_TO: u8 = 0x08;
pub const MOV_DOUBLE_TO: u8 = 0x09;
pub const MOV_REGISTER_TO: u8 = 0x0a;

pub const PLATFORM_CALL: u8 = 0x10;

macro_rules! push_number {
    ( $dest: expr, $num: expr ) => {
        push_bytes!($dest, $num.to_le_bytes())
    };
}

macro_rules! push_string {
    ( $dest: expr, $s: expr ) => {
        push_bytes!($dest, $s.bytes())
    };
}

pub fn add_mov_int(code: &mut Vec<u8>, register: u8, operand: u32) {
    code.push(MOV_INT);
    code.push(register);
    push_number!(code, operand);
}

pub fn add_mov_register(code: &mut Vec<u8>, target_register: u8, source_register: u8) {
    code.push(MOV_REGISTER);
    code.push(target_register);
    code.push(source_register);
}

pub fn add_alloc(code: &mut Vec<u8>, register: u8, size: u32) {
    code.push(ALLOC);
    code.push(register);
    push_number!(code, size);
}

pub fn add_dealloc(code: &mut Vec<u8>, register: u8) {
    code.push(DEALLOC);
    code.push(register);
}

pub fn add_mov_int_to(code: &mut Vec<u8>, register: u8, offset: u32, operand: u32) {
    code.push(MOV_INT_TO);
    code.push(register);
    push_number!(code, offset);
    push_number!(code, operand);
}

pub fn add_platform_call_to(code: &mut Vec<u8>, symbol_name: &String, return_register: u8, arg_registers_length: u8, arg_registers: &Vec<u8>) {
    code.push(PLATFORM_CALL);
    push_number!(code, symbol_name.len() as u32);
    push_string!(code, symbol_name);
    push_number!(code, return_register);
    push_number!(code, arg_registers_length);
    for &b in arg_registers {
        code.push(b);
    }
}
