use crate::vm::dvm_value::DvmValue;
use crate::vm::DmVirtualMachine;

pub fn dm_print(args: Vec<DvmValue>, vm: &mut DmVirtualMachine) -> DvmValue {
    if args.len() != 1 {
        return DvmValue::Unit; // TODO: make exception
    }
    print!("{}", get_string(&args[0]));
    DvmValue::Unit
}

pub fn dm_println(args: Vec<DvmValue>, vm: &mut DmVirtualMachine) -> DvmValue {
    if args.len() != 1 {
        return DvmValue::Unit;
    }
    println!("{}", get_string(&args[0]));
    DvmValue::Unit
}

fn get_string(dvm_value: &DvmValue) -> String {
    match dvm_value {
        DvmValue::Byte(b) => b.to_string(),
        DvmValue::Int(i) => i.to_string(),
        DvmValue::Long(l) => l.to_string(),
        DvmValue::Double(d) => d.to_string(),
        DvmValue::Boolean(b) => b.to_string(),
        DvmValue::Pointer(value_box) => get_string(value_box),
        DvmValue::ByteArray(_) => todo!(),
        DvmValue::IntArray(_) => todo!(),
        DvmValue::LongArray(_) => todo!(),
        DvmValue::DoubleArray(_) => todo!(),
        DvmValue::BooleanArray(_) => todo!(),
        DvmValue::PointerArray(_) => todo!(),
        DvmValue::Unit => String::from("Unit"),
    }
}
